/*
 * Copyright (C) 2008
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.googlecode.sca4idea.psi;

import com.intellij.openapi.util.text.StringUtil;
import com.intellij.psi.PsiElement;
import com.intellij.psi.xml.XmlAttributeValue;
import com.intellij.psi.xml.XmlFile;
import com.intellij.psi.xml.XmlTag;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import com.googlecode.sca4idea.util.SCDLResolver;
import com.googlecode.sca4idea.util.ComponentResolver;

/**
 * PSI Element that provides introspection functionality for components defined by external composites.
 */
public class SCACompositeComponentTagPsiElement extends SCAComponentTagPsiElement {
    private XmlFile file;

    public SCACompositeComponentTagPsiElement(XmlTag underlying, XmlAttributeValue targetAttr, boolean isResource) {
        super(underlying);

        if (StringUtil.isNotEmpty(targetAttr.getText())) {
            file = null;
        } else if (isResource){
            file = SCDLResolver.loadScdlResource(underlying, targetAttr.getText());
        } else {
            file = SCDLResolver.loadScdlLocation(underlying, targetAttr.getText());
        }
    }

    public PsiElement resolveServiceInterface(String serviceInterfaceName) {
        if (file == null) {
            return null;
        }

        // Retrieve all the promotions for the referenced file
        Map<String, XmlTag> promotions = ComponentResolver.getPromotedServices(file);
        return promotions.get(serviceInterfaceName);
    }

    public Object[] getVariantsOfServiceInterfaces() {
        if (file == null) {
            return new Object[0];
        }

        // Retrieve all the promotions for the referenced file
        Map<String, XmlTag> promotions = ComponentResolver.getPromotedServices(file);
        String componentName = getComponentName();
        List<String> result = new ArrayList<String>();
        for (String promotion : promotions.keySet()) {
            result.add(componentName + "/" + promotion);
        }

        return result.toArray();
    }

    public PsiElement resolveReference(String referenceName) {
        if (file == null) {
            return null;
        }

        // Retrieve all the references for the referenced file
        Map<String, XmlTag> references = ComponentResolver.getPromotedReferences(file, true);
        return references.get(referenceName);
    }

    public Object[] getVariantsOfReferences(boolean qualified) {
        if (file == null) {
            return new Object[0];
        }

        // Retrieve all the references for the referenced file
        Map<String, XmlTag> references = ComponentResolver.getPromotedReferences(file, true);
        String componentName = getComponentName();
        List<String> result = new ArrayList<String>();
        for (String reference : references.keySet()) {
            if (qualified) {
                result.add(componentName + "/" + reference);
            } else {
                result.add(reference);
            }
        }

        return result.toArray();
    }

    public Object[] getVariantsOfProperties(boolean qualified) {
        if (file == null) {
            return new Object[0];
        }

        // Retrieve all the references for the referenced file
        Map<String, XmlTag> properties = ComponentResolver.getPromotedProperties(file, true);
        String componentName = getComponentName();
        List<String> result = new ArrayList<String>();
        for (String property : properties.keySet()) {
            if (qualified) {
                result.add(componentName + "/" + property);
            } else {
                result.add(property);
            }
        }

        return result.toArray();
    }

    public PsiElement resolveProperty(String propertyName) {
        if (file == null) {
            return null;
        }

        // Retrieve all the references for the referenced file
        Map<String, XmlTag> references = ComponentResolver.getPromotedProperties(file, true);
        return references.get(propertyName);
    }

    public PsiElement getTarget() {
        return file;
    }
}
